/*!
 * module-loader.js v0.0.10
 * (c) 2014-2020 stwilson
 * Released under the MIT License.
 */
export { to as catchWrap } from 'await-to-js';
import { getUniqueKey, Types, typeCheckExpression } from '@blocks-framework/core';
export * from '@blocks-framework/core';
import { Component as Component$1, Vue, Prop } from 'vue-property-decorator';
export { Prop, Watch } from 'vue-property-decorator';

let Component = (options) => {
    return ComponentFactory(Component$1, options);
};
function ComponentFactory(oldComponentFactory, options) {
    let com = oldComponentFactory(options);
    if (typeof options === 'function') {
        com.options.type = options;
        return com;
    }
    return function (Component) {
        let comObj = com(Component);
        if (typeof Component === 'function')
            comObj.options.type = options;
        return comObj;
    };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

function __metadata(metadataKey, metadataValue) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
}

let isPromise = (obj) => {
    return !!obj && (typeof obj === 'object' || typeof obj === 'function') && typeof obj.then === 'function';
};

const INJECTION = Symbol.for("INJECTION");
function _proxyGetter(proto, key, resolve, doCache) {
    function getter() {
        //let that:any = this;
        if (doCache && !Reflect.hasMetadata(INJECTION, this, key)) {
            Reflect.defineMetadata(INJECTION, resolve(), this, key);
        }
        if (Reflect.hasMetadata(INJECTION, this, key)) {
            return Reflect.getMetadata(INJECTION, this, key);
        }
        else {
            return resolve();
        }
    }
    function setter(newVal) {
        Reflect.defineMetadata(INJECTION, newVal, this, key);
    }
    Object.defineProperty(proto, key, {
        configurable: true,
        enumerable: true,
        get: getter,
        set: setter
    });
}
function makePropertyInjectDecorator(containerFunc, doCache) {
    return function (serviceIdentifier) {
        return function (proto, key) {
            let resolve = () => {
                return containerFunc().get(serviceIdentifier);
            };
            _proxyGetter(proto, key, resolve, doCache);
        };
    };
}
function makePropertyInjectNamedDecorator(containerFunc, doCache) {
    return function (serviceIdentifier, named) {
        return function (proto, key) {
            let resolve = () => {
                return containerFunc().getNamed(serviceIdentifier, named);
            };
            _proxyGetter(proto, key, resolve, doCache);
        };
    };
}
function makePropertyInjectTaggedDecorator(containerFunc, doCache) {
    return function (serviceIdentifier, key, value) {
        return function (proto, propertyName) {
            let resolve = () => {
                return containerFunc().getTagged(serviceIdentifier, key, value);
            };
            _proxyGetter(proto, propertyName, resolve, doCache);
        };
    };
}
function makePropertyMultiInjectDecorator(containerFunc, doCache) {
    return function (serviceIdentifier) {
        return function (proto, key) {
            let resolve = () => {
                return containerFunc().getAll(serviceIdentifier);
            };
            _proxyGetter(proto, key, resolve, doCache);
        };
    };
}

function getDecorators(containerFunc, doCache = true) {
    let lazyInject = makePropertyInjectDecorator(containerFunc, doCache);
    let lazyInjectNamed = makePropertyInjectNamedDecorator(containerFunc, doCache);
    let lazyInjectTagged = makePropertyInjectTaggedDecorator(containerFunc, doCache);
    let lazyMultiInject = makePropertyMultiInjectDecorator(containerFunc, doCache);
    return {
        lazyInject,
        lazyInjectNamed,
        lazyInjectTagged,
        lazyMultiInject
    };
}

//import { Bootstrapper } from "@blocks-framework/core";
class InjectModel {
    constructor() {
        this.Conainter = undefined;
    }
}
let InjectCore = new InjectModel();
let SpecialInject = getDecorators(() => {
    if (InjectCore.Conainter === undefined)
        throw new Error("InjectCore.Conainter is undefined.");
    return InjectCore.Conainter;
});

let Controller = class Controller extends Vue {
    constructor() {
        super();
        this.UniqueKey = "";
        this.switchMode = 0;
        //  this.resourceManager = null;
    }
    created() {
        this.$emit("beforeViewWillEnter", this);
        this.viewWillEnterResult = this.viewWillEnter();
        console.log("assign viewWillEnterResult");
        if ((typeof this.UniqueKey === "undefined" || this.UniqueKey === "") && this.$route && this.$route.meta && this.$route.meta.uniqueKey) {
            this.UniqueKey = this.$route.meta.uniqueKey;
        }
    }
    async mounted() {
        this.$emit("ViewDidEnterStart", this);
        if (isPromise(this.viewDidEnter)) {
            this.viewDidEnterResult = this.viewDidEnter();
            await this.viewDidEnterResult;
        }
        else
            this.viewDidEnter();
        this.viewAnimationEndTime = setTimeout(() => {
            this.viewAnimationEnd();
        }, 800);
    }
    beforeMount() {
        this.$emit("beforeViewDidEnter", this);
        this.beforeViewDidEnter();
    }
    beforeViewDidEnter() {
    }
    viewAnimationEndAndDataReady() {
    }
    async viewAnimationEnd() {
        console.debug("viewAnimationEnd start");
        clearTimeout(this.viewAnimationEndTime);
        if (isPromise(this.viewWillEnterResult)) {
            await this.viewWillEnterResult;
            console.debug("await  viewWillEnterResult");
        }
        console.debug("viewWillEnterResult end");
        if (isPromise(this.viewAnimationEndAndDataReady))
            await this.viewAnimationEndAndDataReady();
        else
            this.viewAnimationEndAndDataReady();
        if (isPromise(this.viewDidEnterResult)) {
            await this.viewDidEnterResult;
            console.debug("await  viewDidEnterResult");
        }
        console.debug("viewAnimationEndAndDataReady end");
        // this.$emit("viewDataReadyFinish")
        let _this = this;
        _this.$emit("viewDataReadyFinish");
        _this.$nextTick(async () => {
            if (_this.$el.querySelectorAll) {
                let allCom = _this.$el.querySelectorAll(".vue-recycle-scroller__item-view");
                for (const com of allCom) {
                    let comChild = com.firstElementChild;
                    if (comChild && comChild.componentOnReady)
                        await comChild.componentOnReady();
                }
            }
            _this.$emit("viewReaderFinish", _this);
        });
    }
    viewWillEnter() {
    }
    viewDidEnter() {
    }
    async switch(switchUniqueKey, switchParams) {
        if (typeof switchParams === undefined || switchParams === null || typeof switchUniqueKey === undefined || switchUniqueKey.uniqueKey === null)
            throw new Error("switchParams must contains uniqueKey.");
        let routeManager = this.routeManager;
        if (routeManager === undefined)
            throw new Error("routeManager is undefined.");
        let uniqueKey = switchUniqueKey.moduleName ? getUniqueKey(switchUniqueKey.moduleName, switchUniqueKey.uniqueKey) :
            getUniqueKey(this.getModuleName(), switchUniqueKey.uniqueKey);
        let route;
        for (let layoutRoutes of routeManager.getRoute()) {
            route = layoutRoutes.children.find((r) => r.uniqueKey === uniqueKey);
            if (route)
                break;
        }
        if (this.switchMode === 0) {
            let dialogParams = Object.assign(true, {
                component: route.component,
            }, switchParams);
            dialogParams.componentProps = Object.assign({ UniqueKey: uniqueKey }, dialogParams.componentProps);
            let dialog = await this.$dialog.create(dialogParams);
            return await dialog.present();
        }
        else {
            let routeReslove, routeReject;
            let routePushPromise = new Promise((resolve, reject) => {
                routeReslove = resolve;
                routeReject = reject;
            });
            this.$router.push({ name: route.name }, () => {
                routeReslove({
                    onDidDismiss: new Promise((resolve, reject) => {
                        this.switchResolve = resolve;
                    })
                });
            }, () => {
                routeReject();
            });
            return await routePushPromise;
        }
    }
    exit(...args) {
        if (this.switchMode === 0) {
            this.$emit("exit", ...args);
        }
        else {
            if (this.switchResolve) {
                this.switchResolve();
            }
        }
    }
    getModuleName() {
        let shell = this.blocksShell; //globalIocManager.get<IBlocksShell>(Types.IBlocksShell);
        if (shell === undefined)
            throw new Error("shell is undefined.");
        let moduleName = shell.typeMapModuleName.get(this.$options.type);
        if (moduleName === undefined)
            throw new Error(this + " not belong to framework.");
        return moduleName;
    }
    getResources(moduleName, resourceType, resourceName) {
        if (this.resourceManager === undefined)
            throw new Error("resourceManager is undefined.");
        let resourceDef = this.resourceManager.getResource(moduleName, resourceType, resourceName);
        if (resourceDef === undefined)
            throw new Error(`Can not found resourceType:${resourceType}, resourceName:${resourceName} in resources.`);
        return resourceDef.resource;
    }
};
__decorate([
    Prop({ type: String, default: "" }),
    __metadata("design:type", String)
], Controller.prototype, "UniqueKey", void 0);
__decorate([
    SpecialInject.lazyInject(Types.IResourceManager),
    __metadata("design:type", Object)
], Controller.prototype, "resourceManager", void 0);
__decorate([
    SpecialInject.lazyInject(Types.IBlocksShell),
    __metadata("design:type", Object)
], Controller.prototype, "blocksShell", void 0);
__decorate([
    SpecialInject.lazyInject(Types.IRouteManager),
    __metadata("design:type", Object)
], Controller.prototype, "routeManager", void 0);
Controller = __decorate([
    Component$1({}),
    __metadata("design:paramtypes", [])
], Controller);
typeCheckExpression.push((exportType) => {
    if (exportType.prototype instanceof Controller && exportType.options.type)
        return exportType.options.type;
});

function asyncCompatible() {
    return function (target, propertyKey, descriptor) {
        let originFunc = descriptor.value;
        descriptor.value = async function (...param) {
            let paramLength = param.length;
            //debugger;
            let actParam = paramLength > 1 ? param.slice(0, paramLength) : param;
            let p = paramLength > 0 ? param[paramLength - 1] : null;
            let returnObj = originFunc.call(this, ...actParam);
            let hasThen = true;
            try {
                hasThen = returnObj.then !== "undefined";
            }
            catch (error) {
                hasThen = false;
            }
            if (hasThen) {
                var result;
                try {
                    result = await returnObj;
                }
                finally {
                    p && p(result);
                }
                // returnObj.then((r: any) => {
                // }).catch((r: any) => {
                //     p && p();
                //     
                // });
            }
            else {
                p && p(returnObj);
            }
        };
    };
}

export { Component, Controller, InjectCore, SpecialInject, asyncCompatible };
